@@ -1,10 +1,7 @@
 import numpy as np
-from sklearn.linear_model import LogisticRegression,Lasso
-from sklearn.neighbors import KNeighborsClassifier,KNeighborsRegressor
-from sklearn.svm import SVC,SVR
-from sklearn.ensemble import RandomForestClassifier,AdaBoostClassifier,RandomForestRegressor,AdaBoostRegressor
 from sklearn.model_selection import train_test_split,GridSearchCV,RandomizedSearchCV
-from sklearn.metrics import accuracy_score,r2_score,confusion_matrix
+from sklearn.metrics import accuracy_score,confusion_matrix
+from sklearn.metrics import r2_score
 import warnings 
 
 def findCombinationsUtil(arr, index, num, reducedNum,size,unique_classes): 
@@ -74,32 +71,54 @@ def find_all_combinations(unique_classes):
 
 class AutoEnClassifier:
     
-    def __init__(self,LR=True,SVC=False,RF=True,AB=False,KNN=False,random_state=0,GridSearch=False,optimize=None,scoring='accuracy'):
-        
+    def __init__(self,LR=True,SVC=False,RF=True,AB=False,LGBM=True,XGB=False,KNN=False,scaling=False,random_state=0,GridSearch=False,optimize=None,scoring='accuracy'):
         self.__LR = LR
         self.__SVC = SVC
         self.__RF = RF
         self.__AB = AB
+        self.__LGBM = LGBM
+        self.__XGB = XGB
         self.__KNN = KNN
+        self.__scaling = scaling
         self.__random_state = random_state
         self.__GridSearch = GridSearch
         self.__optimize = optimize
+        if optimize not in [None,'FP','FN']:
+            raise ValueError("optimize can take only values None,'FP' and 'FN'")
+        if LR + SVC + RF + AB + LGBM + XGB + KNN > 4:
+            warnings.warn('you are using more than 4 models, while ensembling it will take more time than usual')
         if not GridSearch:
-            warnings.warn('model will use RandomizedSearch')
+            warnings.warn('As GridSearch is said to False, model will use RandomizedSearch')
         self.__scoring = scoring
         
         
         
     def fit(self,X_train,y_train,validation_split=0.2,validation_data=False):
-       
         self.__storing_model_names = []
+        
+        
+            
         self.__X_train = X_train
         self.__y_train = y_train
+        if self.__optimize == 'FN' or self.__optimize == 'FP':
+            if len(y_train.value_counts()) != 2:
+                raise ValueError(f'There are should be only 2 Classes for optimizing "{self.__optimize}"')
+            
         if validation_data:
             self.__X_test = validation_data[0]
             self.__y_test = validation_data[1]
+            if self.__scaling:
+                from sklearn.preprocessing import MinMaxScaler
+                self.__scaler = MinMaxScaler()
+                self.__X_train = self.__scaler.fit_transform(self.__X_train)
+                self.__X_test = self.__scaler.transform(self.__X_test)
         else:
-            self.__X_train,self.__X_test,self.__y_train,self.__y_test = train_test_split(X_train,y_train,test_size=validation_split,random_state=self.__random_state)
+            self.__X_train,self.__X_test,self.__y_train,self.__y_test = train_test_split(X_train,y_train,test_size=validation_split,random_state=self.__random_state,stratify=y_train)
+            if self.__scaling:
+                from sklearn.preprocessing import MinMaxScaler
+                self.__scaler = MinMaxScaler()
+                self.__X_train = self.__scaler.fit_transform(self.__X_train)
+                self.__X_test = self.__scaler.transform(self.__X_test)
         
         if self.__LR:
             AutoEnClassifier.LR_model_fit(self,param_grid=None)
@@ -113,6 +132,12 @@ class AutoEnClassifier:
         if self.__AB:
             AutoEnClassifier.AB_model_fit(self,param_grid=None)
             self.__storing_model_names.append('AB_score')
+        if self.__LGBM:
+            AutoEnClassifier.LGBM_model_fit(self,param_grid=None)
+            self.__storing_model_names.append('LGBM_score')
+        if self.__XGB:
+            AutoEnClassifier.XGB_model_fit(self,param_grid=None)
+            self.__storing_model_names.append('XGB_score')
         if self.__KNN:
             AutoEnClassifier.KNN_model_fit(self,list_neighbors=None)
             self.__storing_model_names.append('KNN_score')
@@ -121,7 +146,8 @@ class AutoEnClassifier:
         
         
     def LR_model_fit(self,param_grid=None):
-            LR_model = LogisticRegression()
+            from sklearn.linear_model import LogisticRegression
+            LR_model = LogisticRegression(max_iter=500)
             if param_grid == None:
                 parameters = {'C':[0.1,0.5,1,5,10],
                               'solver':['newton-cg', 'lbfgs', 'sag', 'saga'],
@@ -139,11 +165,12 @@ class AutoEnClassifier:
             print(f'LR_score : {accuracy_score(self.__LR_model.predict(self.__X_test),self.__y_test)}')
             
     def SVC_model_fit(self,param_grid=None):
+            from sklearn.svm import SVC
             SVC_model = SVC(probability=True)
             if param_grid == None:
-                parameters = [{'kernel': ['rbf','poly'],
+                parameters = {'kernel': ['rbf','poly'],
                                'gamma': [1e-3, 1e-4],
-                               'C': [1, 10, 100, 1000]}]
+                               'C': [1, 10, 100,1000]}
                 
                 if self.__GridSearch:
                     self.__SVC_model = GridSearchCV(estimator=SVC_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
@@ -159,9 +186,10 @@ class AutoEnClassifier:
             
             
     def RF_model_fit(self,param_grid=None):
-            RF_model = RandomForestClassifier()
+            from sklearn.ensemble import RandomForestClassifier
+            RF_model = RandomForestClassifier(n_jobs=-1)
             if param_grid == None:
-                parameters = {'n_estimators' :[10,50,100,500],
+                parameters = {'n_estimators' :[10,50,100],
                               'max_depth' : [4,8,10,12,16],
                               'min_samples_leaf' : [0.1, 0.2, 0.3, 0.4, 0.5]
                               }
@@ -181,10 +209,11 @@ class AutoEnClassifier:
             
             
     def AB_model_fit(self,param_grid=None):
+            from sklearn.ensemble import AdaBoostClassifier
             AB_model = AdaBoostClassifier()
             if param_grid == None:
-                parameters = {'n_estimators' :[10,50,100,500],
-                              'learning_rate' : [0.01,0.5,0.1,0.15,0.2],
+                parameters = {'n_estimators' :[10,50,100,150],
+                              'learning_rate' : [0.01,0.05,0.1]
                               }
                 
                 if self.__GridSearch:
@@ -200,9 +229,60 @@ class AutoEnClassifier:
             self.__AB_model.fit(self.__X_train,self.__y_train)
             print(f'AB_score : {accuracy_score(self.__AB_model.predict(self.__X_test),self.__y_test)}')
             
+            
+    def LGBM_model_fit(self,param_grid=None):
+            from lightgbm import LGBMClassifier
+            LGBM_model = LGBMClassifier()
+            if param_grid == None:
+                parameters = {'num_leaves': (15,30,45,60),
+                              'learning_rate' : (0.01,0.05,0.1),
+                              'max_depth': (9, 10,13 ),
+                              'min_child_weight': (1,3,5,7,12)
+                              }
+                
+                if self.__GridSearch:
+                    self.__LGBM_model = GridSearchCV(estimator=LGBM_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__LGBM_model = RandomizedSearchCV(estimator=LGBM_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+            else:
+                if self.__GridSearch:
+                    self.__LGBM_model = GridSearchCV(estimator=LGBM_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__LGBM_model = RandomizedSearchCV(estimator=LGBM_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+            
+            self.__LGBM_model.fit(self.__X_train,self.__y_train)       
+            print(f'LGBM_score : {accuracy_score(self.__LGBM_model.predict(self.__X_test),self.__y_test)}')
+            
+            
+    def XGB_model_fit(self,param_grid=None):
+            from xgboost import XGBClassifier
+            XGB_model = XGBClassifier()
+            if param_grid == None:
+                parameters = {
+                               'min_child_weight':(1,3,5,7),
+                               'gamma':(0,0.1,0.3,0.6),
+                               'subsample':(0.5, 1),
+                               'colsample_bytree':(0.3, 0.5,0.8),
+                               'max_depth': (4,8,12,16)
+                              }
+                
+                if self.__GridSearch:
+                    self.__XGB_model = GridSearchCV(estimator=XGB_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__XGB_model = RandomizedSearchCV(estimator=XGB_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+            else:
+                if self.__GridSearch:
+                    self.__XGB_model = GridSearchCV(estimator=XGB_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__XGB_model = RandomizedSearchCV(estimator=XGB_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                
+            self.__XGB_model.fit(self.__X_train,self.__y_train)
+            print(f'XGB_score : {accuracy_score(self.__XGB_model.predict(self.__X_test),self.__y_test)}')
+            
         
     
     def KNN_model_fit(self,list_neighbors=None):
+            from sklearn.neighbors import KNeighborsClassifier
             if list_neighbors == None:
                 list_neighbors = [3,5,7,9,11,13,15]
                 n_neighbor_score_model = [None,0,None] 
@@ -226,104 +306,104 @@ class AutoEnClassifier:
         
         global combinations
         combinations = []
-        Total_models = self.__LR + self.__SVC + self.__RF + self.__KNN + self.__AB
-        
+        Total_models = self.__LR + self.__SVC + self.__RF + self.__AB + self.__LGBM + self.__XGB + self.__KNN
+        optimize_count = None
         combinations = find_all_combinations(Total_models)
+        combinations = np.array(combinations)
+        all_proba = []
+        count = 1
         
         self.__best_score = [0] + [None] * Total_models
-        count = 1
-        flag_loop = 0
+                    
+        if self.__LR:
+            
+            LR_model_y_predict_proba = self.__LR_model.predict_proba(self.__X_test)
+            all_proba.append(LR_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                        
+        if self.__SVC:
+            
+            SVC_model_y_predict_proba = self.__SVC_model.predict_proba(self.__X_test)
+            all_proba.append(SVC_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                    
+        if self.__RF:
+               
+            RF_model_y_predict_proba = self.__RF_model.predict_proba(self.__X_test)             
+            all_proba.append(RF_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                    
+        if self.__AB:
+            AB_model_y_predict_proba = self.__AB_model.predict_proba(self.__X_test)
+            all_proba.append(AB_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                
+        if self.__LGBM:
+            LGBM_model_y_predict_proba = self.__LGBM_model.predict_proba(self.__X_test)
+            all_proba.append(LGBM_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                
+        if self.__XGB:
+            XGB_model_y_predict_proba = self.__XGB_model.predict_proba(self.__X_test)
+            all_proba.append(XGB_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+                
+        if self.__KNN:
+            KNN_model_y_predict_proba = self.__KNN_model.predict_proba(self.__X_test)
+            all_proba.append(KNN_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+         
+        all_proba = np.array(all_proba)
+                
+        all_proba = np.sum(np.multiply(combinations.T, np.array([all_proba]).T).T,axis=1)
         
-        for comb in combinations: 
-                all_proba = []
-                if self.__LR:
-                    if flag_loop == 0:
-                        LR_loc = count
-                    LR_model_y_predict_proba = self.__LR_model.predict_proba(self.__X_test)
-                    LR_model_y_predict_proba = np.multiply(LR_model_y_predict_proba,comb[LR_loc-1])
-                    all_proba.append(LR_model_y_predict_proba)
-                    if self.__best_score[LR_loc] == None:
-                        count += 1
-                        
-                if self.__SVC:
-                    if flag_loop == 0:
-                        SVC_loc = count
-                    SVC_model_y_predict_proba = self.__SVC_model.predict_proba(self.__X_test)
-                    SVC_model_y_predict_proba = np.multiply(SVC_model_y_predict_proba,comb[SVC_loc-1])
-                    all_proba.append(SVC_model_y_predict_proba)
-                    if self.__best_score[SVC_loc] == None:
-                        count += 1
-                    
-                if self.__RF:
-                    if flag_loop == 0:
-                        RF_loc = count
-                    RF_model_y_predict_proba = self.__RF_model.predict_proba(self.__X_test)
-                    RF_model_y_predict_proba = np.multiply(RF_model_y_predict_proba,comb[RF_loc-1])
-                    all_proba.append(RF_model_y_predict_proba)
-                    if self.__best_score[RF_loc] == None:
-                        count += 1
-                    
-                if self.__AB:
-                    if flag_loop == 0:
-                        AB_loc = count
-                    AB_model_y_predict_proba = self.__AB_model.predict_proba(self.__X_test)
-                    AB_model_y_predict_proba = np.multiply(AB_model_y_predict_proba,comb[AB_loc-1])
-                    all_proba.append(AB_model_y_predict_proba)
-                    if self.__best_score[AB_loc] == None:
-                        count += 1
+        for proba,comb in zip(all_proba,combinations):
                 
-                if self.__KNN:
-                    if flag_loop == 0:
-                        KNN_loc = count
-                    KNN_model_y_predict_proba = self.__KNN_model.predict_proba(self.__X_test)
-                    KNN_model_y_predict_proba = np.multiply(KNN_model_y_predict_proba,comb[KNN_loc-1])
-                    all_proba.append(KNN_model_y_predict_proba)
-                    if self.__best_score[KNN_loc] == None:
-                        count += 1
-                    
-                y_predict = np.sum(all_proba,axis=0)
-                
-                y_predict = np.argmax(y_predict,axis=1)
+            y_predict = np.argmax(proba,axis=1)
                 
                 
-                latest_score = accuracy_score(self.__y_test,y_predict)
+            latest_score = accuracy_score(self.__y_test,y_predict)
                 
                 
-                if latest_score > self.__best_score[0]:
-                    if flag_loop==0 and self.__optimize == 'FP':
-                        optimize_count = confusion_matrix(self.__y_test,y_predict)[1][0]
-                        
-                    if flag_loop==0 and self.__optimize == 'FN':
-                        optimize_count = confusion_matrix(self.__y_test,y_predict)[0][1]
+            if latest_score > self.__best_score[0]:
                         
                     self.__best_score[0] = latest_score
                     for i in range(0,len(comb)):
                         self.__best_score[i+1] = comb[i]
                         
-                elif latest_score == self.__best_score[0] and self.__optimize == 'FP':
-                    
-                    FP_count = confusion_matrix(self.__y_test,y_predict)[1][0]
-                    if FP_count < optimize_count:
-                        print(f'optimized FP from {optimize_count} to {FP_count}')
-                        optimize_count = FP_count
-                        self.__best_score[0] = latest_score
-                    for i in range(0,len(comb)):
-                        self.__best_score[i+1] = comb[i]
+                    if self.__optimize == 'FP' and optimize_count==None:
+                        optimize_count = confusion_matrix(self.__y_test,y_predict)[1][0]
+                    elif self.__optimize == 'FN' and optimize_count==None:
+                        optimize_count = confusion_matrix(self.__y_test,y_predict)[0][1]
                         
-                elif latest_score == self.__best_score[0] and self.__optimize == 'FN':
-                    
-                    FN_count = confusion_matrix(self.__y_test,y_predict)[0][1]
-                    if FN_count < optimize_count:
-                        print(f'optimized FN from {optimize_count} to {FN_count}')
-                        optimize_count = FN_count
-                        self.__best_score[0] = latest_score
-                    for i in range(0,len(comb)):
-                        self.__best_score[i+1] = comb[i]
-                
                         
+            elif latest_score == self.__best_score[0] and self.__optimize == 'FP':
+                FP_count = confusion_matrix(self.__y_test,y_predict)[1][0]
+                    
+                if FP_count < optimize_count:
+                    print(f'optimized FP count from {optimize_count} to {FP_count}')
+                    optimize_count = FP_count
+                    self.__best_score[0] = latest_score
+                for i in range(0,len(comb)):
+                    self.__best_score[i+1] = comb[i]
+                        
+            elif latest_score == self.__best_score[0] and self.__optimize == 'FN':
+                FN_count = confusion_matrix(self.__y_test,y_predict)[0][1]
                 
-                if flag_loop == 0:
-                    flag_loop = 1
+                if FN_count < optimize_count:
+                    print(f'optimized FN count from {optimize_count} to {FN_count}')
+                    optimize_count = FN_count
+                    self.__best_score[0] = latest_score
+                for i in range(0,len(comb)):
+                    self.__best_score[i+1] = comb[i]
+            
                     
         print(f'AutoEn_score : {self.__best_score[0]}')
         for i in range(len(self.__storing_model_names)):
@@ -332,6 +412,9 @@ class AutoEnClassifier:
             
         
     def predict(self,X_test):
+        
+        if self.__scaling:
+                X_test = self.__scaler.transform(X_test)
         all_proba = []
         count = 1
         try:
@@ -360,6 +443,18 @@ class AutoEnClassifier:
                 all_proba.append(AB_model_y_predict_proba)
                 count+=1
                 
+            if self.__LGBM:
+                LGBM_model_y_predict_proba = self.__LGBM_model.predict_proba(X_test)
+                LGBM_model_y_predict_proba = np.multiply(LGBM_model_y_predict_proba,self.__best_score[count])
+                all_proba.append(LGBM_model_y_predict_proba)
+                count+=1
+                
+            if self.__XGB:
+                XGB_model_y_predict_proba = self.__XGB_model.predict_proba(X_test)
+                XGB_model_y_predict_proba = np.multiply(XGB_model_y_predict_proba,self.__best_score[count])
+                all_proba.append(XGB_model_y_predict_proba)
+                count+=1
+                
             if self.__KNN:
                 KNN_model_y_predict_proba = self.__KNN_model.predict_proba(X_test)
                 KNN_model_y_predict_proba = np.multiply(KNN_model_y_predict_proba,self.__best_score[count])
@@ -370,7 +465,12 @@ class AutoEnClassifier:
            
             
         except AttributeError:
-            print('model not fitted yet')
+            from sklearn.exceptions import NotFittedError
+            raise NotFittedError('model not fitted yet')
+            return None
+        
+        except:
+            print('something went wrong')
             return None
         
         
@@ -380,37 +480,53 @@ class AutoEnClassifier:
         return y_predict
     
 class AutoEnRegressor:
-    def __init__(self,LR=True,SVR=False,RF=True,AB=False,KNN=False,random_state=0,GridSearch=False,scoring='r2'):
+    def __init__(self,LA=True,SVR=False,RF=True,AB=False,LGBM=True,XGB=False,KNN=False,scaling=False,random_state=0,GridSearch=False,scoring='r2'):
         
-        
-        self.__LR = LR
+        self.__LA = LA
         self.__SVR = SVR
         self.__RF = RF
         self.__AB = AB
+        self.__LGBM = LGBM
+        self.__XGB = XGB
         self.__KNN = KNN
+        self.__scaling = scaling
         self.__random_state = random_state
         self.__GridSearch = GridSearch
+        if LA + SVR + RF + AB + LGBM + XGB + KNN > 4:
+            warnings.warn('you are using more than 4 models, while ensembling it will take more time than usual')
         if not GridSearch:
-            warnings.warn('model will use RandomizedSearch')
+            warnings.warn('As GridSearch is said to False, model will use RandomizedSearch')
         self.__scoring = scoring
         
             
 
         
     def fit(self,X_train,y_train,validation_split=0.2,validation_data=False):
-       
         self.__storing_model_names = []
+        
         self.__X_train = X_train
         self.__y_train = y_train
+        
         if validation_data:
             self.__X_test = validation_data[0]
             self.__y_test = validation_data[1]
+            if self.__scaling:
+                from sklearn.preprocessing import MinMaxScaler
+                self.__scaler = MinMaxScaler()
+                self.__X_train = self.__scaler.fit_transform(self.__X_train)
+                self.__X_test = self.__scaler.transform(self.__X_test)
         else:
             self.__X_train,self.__X_test,self.__y_train,self.__y_test = train_test_split(X_train,y_train,test_size=validation_split,random_state=self.__random_state)
+            if self.__scaling:
+                from sklearn.preprocessing import MinMaxScaler
+                self.__scaler = MinMaxScaler()
+                self.__X_train = self.__scaler.fit_transform(self.__X_train)
+                self.__X_test = self.__scaler.transform(self.__X_test)
+            
         
-        if self.__LR:
-            AutoEnRegressor.LR_model_fit(self,param_grid=None)
-            self.__storing_model_names.append('LR_score')
+        if self.__LA:
+            AutoEnRegressor.LA_model_fit(self,param_grid=None)
+            self.__storing_model_names.append('LA_score')
         if self.__SVR:
             AutoEnRegressor.SVR_model_fit(self,param_grid=None)
             self.__storing_model_names.append('SVR_score')
@@ -420,6 +536,12 @@ class AutoEnRegressor:
         if self.__AB:
             AutoEnRegressor.AB_model_fit(self,param_grid=None)
             self.__storing_model_names.append('AB_score')
+        if self.__LGBM:
+            AutoEnRegressor.LGBM_model_fit(self,param_grid=None)
+            self.__storing_model_names.append('LGBM_score')
+        if self.__XGB:
+            AutoEnRegressor.XGB_model_fit(self,param_grid=None)
+            self.__storing_model_names.append('XGB_score')
         if self.__KNN:
             AutoEnRegressor.KNN_model_fit(self,list_neighbors=None)
             self.__storing_model_names.append('KNN_score')
@@ -427,29 +549,30 @@ class AutoEnRegressor:
         AutoEnRegressor.find_best(self)
         
         
-    def LR_model_fit(self,param_grid=None):
-            LR_model = Lasso()
+    def LA_model_fit(self,param_grid=None):
+            from sklearn.linear_model import Lasso
+            LA_model = Lasso(max_iter=500)
             if param_grid == None:
-                parameters = {'alpha':[0.01,0.5,1,2,5]
+                parameters = {'alpha':[1,2,5,10]
                               }
                 if self.__GridSearch:
-                    self.__LR_model = GridSearchCV(estimator=LR_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                    self.__LA_model = GridSearchCV(estimator=LA_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
                 else:
-                    self.__LR_model = RandomizedSearchCV(estimator=LR_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                    self.__LA_model = RandomizedSearchCV(estimator=LA_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
             else:
                 if self.__GridSearch:
-                    self.__LR_model = GridSearchCV(estimator=LR_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                    self.__LA_model = GridSearchCV(estimator=LA_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
                 else:
-                    self.__LR_model = RandomizedSearchCV(estimator=LR_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
-            self.__LR_model.fit(self.__X_train,self.__y_train)
-            print(f'LR_score : {r2_score(self.__y_test,self.__LR_model.predict(self.__X_test))}')
+                    self.__LA_model = RandomizedSearchCV(estimator=LA_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+            self.__LA_model.fit(self.__X_train,self.__y_train)
+            print(f'LA_score : {r2_score(self.__y_test,self.__LA_model.predict(self.__X_test))}')
             
     def SVR_model_fit(self,param_grid=None):
+            from sklearn.svm import SVR
             SVR_model = SVR()
             if param_grid == None:
-                parameters = [{'kernel': ['rbf','poly'],
-                               'gamma': [1e-3, 1e-4],
-                               'C': [1, 10, 100, 1000]}]
+                parameters = {'kernel': ['rbf','poly'],
+                               'C': [1, 10, 100,1000]}
                 
                 if self.__GridSearch:
                     self.__SVR_model = GridSearchCV(estimator=SVR_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
@@ -465,9 +588,10 @@ class AutoEnRegressor:
             
             
     def RF_model_fit(self,param_grid=None):
+            from sklearn.ensemble import RandomForestRegressor
             RF_model = RandomForestRegressor()
             if param_grid == None:
-                parameters = {'n_estimators' :[10,50,100,500],
+                parameters = {'n_estimators' :[10,50,100,150],
                               'max_depth' : [4,8,10,12,16],
                               'min_samples_leaf' : [0.1, 0.2, 0.3, 0.4, 0.5]
                               }
@@ -487,10 +611,11 @@ class AutoEnRegressor:
             
             
     def AB_model_fit(self,param_grid=None):
+            from sklearn.ensemble import AdaBoostRegressor
             AB_model = AdaBoostRegressor()
             if param_grid == None:
-                parameters = {'n_estimators' :[10,50,100,500],
-                              'learning_rate' : [0.01,0.5,0.1,0.15,0.2],
+                parameters = {'n_estimators' :[10,50,100],
+                              'learning_rate' : [0.01,0.05,0.1]
                               }
                 
                 if self.__GridSearch:
@@ -506,9 +631,58 @@ class AutoEnRegressor:
             self.__AB_model.fit(self.__X_train,self.__y_train)
             print(f'AB_score : {r2_score(self.__y_test,self.__AB_model.predict(self.__X_test))}')
             
-        
+    def LGBM_model_fit(self,param_grid=None):
+            from lightgbm import LGBMRegressor
+            LGBM_model = LGBMRegressor()
+            if param_grid == None:
+                parameters = {'num_leaves': (15,30,45,60),
+                              'learning_rate' : (0.01,0.05,0.1),
+                              'max_depth': (9, 10,13 ),
+                              'min_child_weight': (1,3,5,7,12)
+                              }
+                
+                if self.__GridSearch:
+                    self.__LGBM_model = GridSearchCV(estimator=LGBM_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__LGBM_model = RandomizedSearchCV(estimator=LGBM_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+            else:
+                if self.__GridSearch:
+                    self.__LGBM_model = GridSearchCV(estimator=LGBM_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__LGBM_model = RandomizedSearchCV(estimator=LGBM_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                
+            self.__LGBM_model.fit(self.__X_train,self.__y_train)
+            print(f'LGBM_score : {r2_score(self.__y_test,self.__LGBM_model.predict(self.__X_test))}')
+            
+            
+    def XGB_model_fit(self,param_grid=None):
+            from xgboost import XGBRegressor
+            XGB_model = XGBRegressor()
+            if param_grid == None:
+                parameters = {
+                               'min_child_weight':(1,3,5,7),
+                               'gamma':(0,0.1,0.3,0.6),
+                               'subsample':(0.5, 1),
+                               'colsample_bytree':(0.3, 0.5,0.8),
+                               'max_depth': (4,8,12,16)
+                              }
+                
+                if self.__GridSearch:
+                    self.__XGB_model = GridSearchCV(estimator=XGB_model, param_grid=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__XGB_model = RandomizedSearchCV(estimator=XGB_model, param_distributions=parameters, cv=5,scoring=self.__scoring,n_jobs=-1)
+            else:
+                if self.__GridSearch:
+                    self.__XGB_model = GridSearchCV(estimator=XGB_model, param_grid=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                else:
+                    self.__XGB_model = RandomizedSearchCV(estimator=XGB_model, param_distributions=param_grid, cv=5,scoring=self.__scoring,n_jobs=-1)
+                
+            self.__XGB_model.fit(self.__X_train,self.__y_train)
+            print(f'XGB_score : {r2_score(self.__y_test,self.__XGB_model.predict(self.__X_test))}')
+            
     
     def KNN_model_fit(self,list_neighbors=None):
+            from sklearn.neighbors import KNeighborsRegressor
             if list_neighbors == None:
                 list_neighbors = [3,5,7,9,11,13,15]
                 n_neighbor_score_model = [None,0,None] 
@@ -531,74 +705,73 @@ class AutoEnRegressor:
         
         global combinations
         combinations = []
-        Total_models = self.__LR + self.__SVR + self.__RF  + self.__AB + self.__KNN
-        
-        combinations = find_all_combinations(Total_models)
+        Total_models = self.__LA + self.__SVR + self.__RF + self.__AB + self.__LGBM + self.__XGB + self.__KNN 
+        combinations = np.array(find_all_combinations(Total_models))
+        all_proba = []
+        count = 1
         
         self.__best_score = [0] + [None] * Total_models
-        count = 1
-        flag_loop = 0
-        
-        for comb in combinations: 
-                all_proba = []
-                if self.__LR:
-                    if flag_loop == 0:
-                        LR_loc = count
-                    LR_model_y_predict = self.__LR_model.predict(self.__X_test)
-                    LR_model_y_predict = np.multiply(LR_model_y_predict,comb[LR_loc-1])
-                    all_proba.append(LR_model_y_predict)
-                    if self.__best_score[LR_loc] == None:
-                        count += 1
+                    
+        if self.__LA:
+            
+            LA_model_y_predict_proba = self.__LA_model.predict(self.__X_test)
+            all_proba.append(LA_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
                         
-                if self.__SVR:
-                    if flag_loop == 0:
-                        SVR_loc = count
-                    SVR_model_y_predict = self.__SVR_model.predict(self.__X_test)
-                    SVR_model_y_predict = np.multiply(SVR_model_y_predict,comb[SVR_loc-1])
-                    all_proba.append(SVR_model_y_predict)
-                    if self.__best_score[SVR_loc] == None:
-                        count += 1
+        if self.__SVR:
+            
+            SVR_model_y_predict_proba = self.__SVR_model.predict(self.__X_test)
+            all_proba.append(SVR_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
                     
-                if self.__RF:
-                    if flag_loop == 0:
-                        RF_loc = count
-                    RF_model_y_predict = self.__RF_model.predict(self.__X_test)
-                    RF_model_y_predict = np.multiply(RF_model_y_predict,comb[RF_loc-1])
-                    all_proba.append(RF_model_y_predict)
-                    if self.__best_score[RF_loc] == None:
-                        count += 1
+        if self.__RF:
+               
+            RF_model_y_predict_proba = self.__RF_model.predict(self.__X_test)             
+            all_proba.append(RF_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
                     
-                if self.__AB:
-                    if flag_loop == 0:
-                        AB_loc = count
-                    AB_model_y_predict = self.__AB_model.predict(self.__X_test)
-                    AB_model_y_predict = np.multiply(AB_model_y_predict,comb[AB_loc-1])
-                    all_proba.append(AB_model_y_predict)
-                    if self.__best_score[AB_loc] == None:
-                        count += 1
+        if self.__AB:
+            AB_model_y_predict_proba = self.__AB_model.predict(self.__X_test)
+            all_proba.append(AB_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
                 
-                if self.__KNN:
-                    if flag_loop == 0:
-                        KNN_loc = count
-                    KNN_model_y_predict = self.__KNN_model.predict(self.__X_test)
-                    KNN_model_y_predict = np.multiply(KNN_model_y_predict,comb[KNN_loc-1])
-                    all_proba.append(KNN_model_y_predict)
-                    if self.__best_score[KNN_loc] == None:
-                        count += 1
-                    
-                y_predict = np.sum(all_proba,axis=0)
+        if self.__LGBM:
+            LGBM_model_y_predict_proba = self.__LGBM_model.predict(self.__X_test)
+            all_proba.append(LGBM_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
                 
-                          
-                latest_score = r2_score(self.__y_test,y_predict)
-          
-                if latest_score > self.__best_score[0]:
+        if self.__XGB:
+            XGB_model_y_predict_proba = self.__XGB_model.predict(self.__X_test)
+            all_proba.append(XGB_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+
+                
+        if self.__KNN:
+            KNN_model_y_predict_proba = self.__KNN_model.predict(self.__X_test)
+            all_proba.append(KNN_model_y_predict_proba)
+            if self.__best_score[count] == None:
+                count += 1
+         
+        all_proba = np.array(all_proba)
+                
+        all_proba = np.sum(np.multiply(combinations.T ,np.array([all_proba]).T  ).T,axis=1)
+        
+        for y_predict,comb in zip(all_proba,combinations):
+                
+            latest_score = r2_score(self.__y_test,y_predict)
+                
+            if latest_score > self.__best_score[0]:
+                        
                     self.__best_score[0] = latest_score
                     for i in range(0,len(comb)):
                         self.__best_score[i+1] = comb[i]
-                        
-                
-                if flag_loop == 0:
-                    flag_loop = 1
+                    
                     
         print(f'AutoEn_score : {self.__best_score[0]}')
         for i in range(len(self.__storing_model_names)):
@@ -607,14 +780,17 @@ class AutoEnRegressor:
             
         
     def predict(self,X_test):
+        
+        if self.__scaling:
+                X_test = self.__scaler.transform(X_test)
         all_proba = []
         count = 1
         try:
         
-            if self.__LR:
-                LR_model_y_predict = self.__LR_model.predict(X_test)
-                LR_model_y_predict = np.multiply(LR_model_y_predict,self.__best_score[count])
-                all_proba.append(LR_model_y_predict)
+            if self.__LA:
+                LA_model_y_predict = self.__LA_model.predict(X_test)
+                LA_model_y_predict = np.multiply(LA_model_y_predict,self.__best_score[count])
+                all_proba.append(LA_model_y_predict)
                 count+=1
                 
             if self.__SVR:
@@ -634,7 +810,19 @@ class AutoEnRegressor:
                 AB_model_y_predict = np.multiply(AB_model_y_predict,self.__best_score[count])
                 all_proba.append(AB_model_y_predict)
                 count+=1
+            
+            if self.__LGBM:
+                LGBM_model_y_predict_proba = self.__LGBM_model.predict(X_test)
+                LGBM_model_y_predict_proba = np.multiply(LGBM_model_y_predict_proba,self.__best_score[count])
+                all_proba.append(LGBM_model_y_predict_proba)
+                count+=1
                 
+            if self.__XGB:
+                XGB_model_y_predict_proba = self.__XGB_model.predict(X_test)
+                XGB_model_y_predict_proba = np.multiply(XGB_model_y_predict_proba,self.__best_score[count])
+                all_proba.append(XGB_model_y_predict_proba)
+                count+=1
+             
             if self.__KNN:
                 KNN_model_y_predict = self.__KNN_model.predict(X_test)
                 KNN_model_y_predict = np.multiply(KNN_model_y_predict,self.__best_score[count])
@@ -645,7 +833,12 @@ class AutoEnRegressor:
            
             
         except AttributeError:
-            print('model not fitted yet')
+            from sklearn.exceptions import NotFittedError
+            raise NotFittedError('model not fitted yet')
+            return None
+        
+        except:
+            print('something went wrong')
             return None
 
         
